@@ -27,6 +27,7 @@ import os
 import time
 
 from urllib import unquote
+from xml.sax.saxutils import escape
 from gettext import gettext as _
 from gettext import ngettext
 
@@ -35,10 +36,12 @@ from pitivi.ui.pathwalker import PathWalker
 from pitivi.ui.filelisterrordialog import FileListErrorDialog
 from pitivi.configure import get_pixmap_dir
 from pitivi.signalgroup import SignalGroup
-from pitivi.stream import VideoStream, AudioStream, TextStream
+from pitivi.stream import VideoStream, AudioStream, TextStream, \
+        MultimediaStream
 from pitivi.settings import GlobalSettings
 from pitivi.utils import beautify_length
 from pitivi.log.loggable import Loggable
+from pitivi.sourcelist import SourceListError
 
 GlobalSettings.addConfigSection('clip-library')
 GlobalSettings.addConfigOption('lastImportFolder',
@@ -55,7 +58,8 @@ GlobalSettings.addConfigOption('closeImportDialog',
  COL_INFOTEXT,
  COL_FACTORY,
  COL_URI,
- COL_LENGTH) = range(5)
+ COL_LENGTH,
+ COL_SEARCH_TEXT) = range(6)
 
 ui = '''
 <ui>
@@ -88,10 +92,7 @@ def beautify_stream(stream):
             templ = ngettext("<b>Audio:</b> %d channel at %d <i>Hz</i> (%d <i>bits</i>)",
                     "<b>Audio:</b> %d channels at %d <i>Hz</i> (%d <i>bits</i>)",
                     stream.channels)
-            try:
-                templ = templ % (stream.channels, stream.rate, stream.width)
-            except TypeError:
-                import pdb; pdb.set_trace()
+            templ = templ % (stream.channels, stream.rate, stream.width)
             return templ
 
         return _("<b>Unknown Audio format:</b> %s") % stream.audiotype
@@ -114,15 +115,18 @@ def beautify_stream(stream):
     raise NotImplementedError
 
 def beautify_factory(factory):
-    ranks = {VideoStream: 0, AudioStream: 1}
+    ranks = {VideoStream: 0, AudioStream: 1, TextStream: 2, MultimediaStream: 3}
     def stream_sort_key(stream):
         return ranks[type(stream)]
 
     streams = factory.getOutputStreams()
     streams.sort(key=stream_sort_key)
-    return ("<b>" + unquote(factory.displayname) + "</b>\n" +
+    return ("<b>" + escape(unquote(factory.name)) + "</b>\n" +
         "\n".join((beautify_stream(stream) for stream in streams)))
 
+def factory_name(factory):
+    return escape(unquote(factory.name))
+
 class SourceList(gtk.VBox, Loggable):
     """ Widget for listing sources """
 
@@ -139,7 +143,8 @@ class SourceList(gtk.VBox, Loggable):
 
         # Store
         # icon, infotext, objectfactory, uri, length
-        self.storemodel = gtk.ListStore(gtk.gdk.Pixbuf, str, object, str, str)
+        self.storemodel = gtk.ListStore(gtk.gdk.Pixbuf, str, object, str, str,
+            str)
 
         self.set_border_width(5)
         self.set_spacing(6)
@@ -181,6 +186,7 @@ class SourceList(gtk.VBox, Loggable):
         self.treeview.connect("row-activated", self._rowActivatedCb)
         self.treeview.set_property("rules_hint", True)
         self.treeview.set_headers_visible(False)
+        self.treeview.set_property("search_column", COL_SEARCH_TEXT)
         tsel = self.treeview.get_selection()
         tsel.set_mode(gtk.SELECTION_MULTIPLE)
         tsel.connect("changed", self._treeSelectionChanged)
@@ -247,7 +253,8 @@ class SourceList(gtk.VBox, Loggable):
         # Connect to project.  We must remove and reset the callbacks when
         # changing project.
         self.project_signals = SignalGroup()
-        self._connectToProject(self.app.current)
+        self.app.connect("new-project-created",
+            self._newProjectCreatedCb)
         self.app.connect("new-project-loaded",
             self._newProjectLoadedCb)
         self.app.connect("new-project-failed",
@@ -298,6 +305,7 @@ class SourceList(gtk.VBox, Loggable):
 
         actiongroup = gtk.ActionGroup("sourcelistpermanent")
         actiongroup.add_actions(actions)
+        actiongroup.get_action("ImportSources").props.is_important = True
         uiman.insert_action_group(actiongroup, 0)
 
         self.selection_actions = gtk.ActionGroup("sourcelistselection")
@@ -316,14 +324,16 @@ class SourceList(gtk.VBox, Loggable):
         self._removeSources()
 
     def _insertEndCb(self, unused_action):
+        self.app.action_log.begin("add clip")
         timeline = self.app.current.timeline
         sources = self.app.current.sources
         start = timeline.duration
         for uri in self.getSelectedItems():
-            factory = sources[uri]
+            factory = sources.getUri(uri)
             source = timeline.addSourceFactory(factory)
             source.setStart(start)
             start += source.duration
+        self.app.action_log.commit()
 
     def _getIcon(self, iconname, alternate):
         icontheme = gtk.icon_theme_get_default()
@@ -346,13 +356,13 @@ class SourceList(gtk.VBox, Loggable):
 
         """
         self.project_signals.connect(
-            project.sources, "file_added", None, self._fileAddedCb)
+            project.sources, "source-added", None, self._sourceAddedCb)
         self.project_signals.connect(
-            project.sources, "file_removed", None, self._fileRemovedCb)
+            project.sources, "source-removed", None, self._sourceRemovedCb)
         self.project_signals.connect(
             project.sources, "discovery-error", None, self._discoveryErrorCb)
-        self.project_signals.connect(
-            project.sources, "missing-plugins", None, self._missingPluginsCb)
+        #self.project_signals.connect(
+        #    project.sources, "missing-plugins", None, self._missingPluginsCb)
         self.project_signals.connect(
             project.sources, "ready", None, self._sourcesStoppedImportingCb)
         self.project_signals.connect(
@@ -464,17 +474,18 @@ class SourceList(gtk.VBox, Loggable):
         self.storemodel.append([thumbnail,
             beautify_factory(factory),
             factory,
-            factory.name,
-            duration])
+            factory.uri,
+            duration,
+            factory_name(factory)])
         self._displayTreeView()
 
     # sourcelist callbacks
 
-    def _fileAddedCb(self, unused_sourcelist, factory):
+    def _sourceAddedCb(self, unused_sourcelist, factory):
         """ a file was added to the sourcelist """
         self._addFactory(factory)
 
-    def _fileRemovedCb(self, unused_sourcelist, uri):
+    def _sourceRemovedCb(self, sourcelist, uri, factory):
         """ the given uri was removed from the sourcelist """
         # find the good line in the storemodel and remove it
         model = self.storemodel
@@ -549,7 +560,7 @@ class SourceList(gtk.VBox, Loggable):
         selected.sort(reverse=True)
         for path in selected:
             uri = model[path][COL_URI]
-            del self.app.current.sources[uri]
+            self.app.current.sources.removeUri(uri)
 
     ## UI Button callbacks
 
@@ -569,7 +580,7 @@ class SourceList(gtk.VBox, Loggable):
             return
         path = paths[0]
         factory = model[path][COL_FACTORY]
-        self.debug("Let's play %s", factory.name)
+        self.debug("Let's play %s", factory.uri)
         self.emit('play', factory)
 
     _dragStarted = False
@@ -582,7 +593,10 @@ class SourceList(gtk.VBox, Loggable):
         result = treeview.get_path_at_pos(int(event.x), int(event.y))
         if result:
             path = result[0]
-            return treeview.get_selection().path_is_selected(path)
+            selection = treeview.get_selection()
+
+            return selection.path_is_selected(path) and selection.count_selected_rows() > 1
+
         return False
 
     def _nothingUnderMouse(self, treeview, event):
@@ -652,19 +666,19 @@ class SourceList(gtk.VBox, Loggable):
         factory = self.storemodel[path][COL_FACTORY]
         self.emit('play', factory)
 
-    def _newProjectLoadedCb(self, unused_pitivi, project):
+    def _newProjectCreatedCb(self, app, project):
         # clear the storemodel
         self.storemodel.clear()
         self._connectToProject(project)
-        # synchronize the storemodel with the new project's sourcelist
-        if project.loaded:
-            for uri, factory in project.sources:
-                self.log("loading uri %s", uri)
-                self._addFactory(factory)
-        else:
-            if not self.infostub.showing:
-                self.pack_start(self.infostub, expand=False)
-                self.infostub.startingImport()
+
+
+    def _newProjectLoadingCb(self, unused_pitivi, uri):
+        if not self.infostub.showing:
+            self.pack_start(self.infostub, expand=False)
+            self.infostub.startingImport()
+
+    def _newProjectLoadedCb(self, unused_pitivi, project):
+        pass
 
     def _newProjectFailedCb(self, unused_pitivi, unused_reason,
         unused_uri):
@@ -700,7 +714,11 @@ class SourceList(gtk.VBox, Loggable):
                 directories = [incoming]
         if directories:
             self.addFolders(directories)
-        self.addUris(filenames)
+        try:
+            self.addUris(filenames)
+        except SourceListError:
+            # filenames already present in the sourcelist
+            pass
 
     def _dndTreeBeginCb(self, unused_widget, context):
         self.info("tree drag_begin")
@@ -742,8 +760,8 @@ class InfoStub(gtk.HBox, Loggable):
         self.errors = []
         self.showing = False
         self._importingmessage = _("Importing clips...")
-        self._errorsmessage = _("Error(s) occured while importing")
-        self._errormessage = _("An error occured while importing")
+        self._errorsmessage = _("Error(s) occurred while importing")
+        self._errormessage = _("An error occurred while importing")
         self._makeUI()
 
     def _makeUI(self):
